export class SerializationStrategy {
    evaluateTerm(value, options) {
        switch (value.termType) {
            case 'Literal':
                return this.evaluateLiteral(value, options);
            case 'NamedNode':
                return this.evaluateNamedNode(value, options);
            case 'BlankNode':
                return this.evaluateBlankNode(value);
            case 'Variable':
                return this.evaluateVariable(value);
        }
        return {
            value: '',
            prefixes: [],
        };
    }
}
export class TemplateResult {
    constructor({ strings, values, tag, strategy, defaultOptions, }) {
        this.strings = strings;
        this.values = values;
        this._tag = tag;
        this.__strategy = strategy;
        this.__defaultOptions = defaultOptions;
    }
    toString(options) {
        let actualOptions = this.__defaultOptions;
        if (options) {
            actualOptions = { ...actualOptions, ...options };
        }
        const { value, prefixes } = this._toPartialString(actualOptions);
        return this.__strategy.getFinalString(value, prefixes, actualOptions);
    }
    _toPartialString(options) {
        const prefixes = new Set();
        const l = this.strings.length - 1;
        let result = '';
        for (let i = 0; i < l; i++) {
            let partialResult = {
                value: '',
                prefixes: [],
            };
            result += this.strings[i];
            const value = this.values[i];
            if (!value)
                continue;
            if (typeof value === 'string') {
                result += value;
            }
            else if (value instanceof TemplateResult) {
                partialResult = value._toPartialString(options);
            }
            else if ('subject' in value) {
                partialResult = this.__strategy.evaluateQuad(value, options);
            }
            else if ('match' in value) {
                partialResult = this.__strategy.evaluateDataset(value, options);
            }
            else {
                partialResult = this.__strategy.evaluateTerm(value, options);
            }
            result += partialResult.value;
            [...partialResult.prefixes].forEach(prefix => prefixes.add(prefix));
        }
        result += this.strings[l];
        return {
            value: result,
            prefixes,
        };
    }
}
